from asyncore import ExitNow
from types import WrapperDescriptorType
from dash import Input, Output,State, callback,no_update,clientside_callback,ClientsideFunction

import datetime
import flask
import os
import dash_bootstrap_components as dbc
from functools import wraps
from drive_screen.alarm_panel import *
from drive_screen.style import *
from drive_screen.alarm_api import user_list_api
from drive_screen.alarm_panel_callback_decorator import *

import inspect
import functools
import dynamic_function_loader
from copy import copy
from retry import retry


    


def panel_callback(app,drive_alarm,creator_list,SERVER_HOST):
    try:
        app.config.external_stylesheets.append('/static/style.css')
        @app.server.route('/static/<resource>')
        def serve_static(resource):
            print('resource',resource)
            STATIC_PATH = os.path.join(os.getcwd(), 'drive_screen\static')

            print('STATIC_PATH',STATIC_PATH)
            return flask.send_from_directory(STATIC_PATH, resource)
    except:
        pass

    try:

   
        # app.config.external_stylesheets.append('/static/style.css')
        # @authenticate_admin
       

        @app.callback(
            Output('card_output', 'children'),
            Output("tips_button", "children"),
            Input('apply', 'n_clicks'),
            Input('interval_1m', 'n_intervals')
            # prevent_initial_call=True
        )
        def card_output_fun(n1,intervals):
            """
            更新card
            """
            # print('更新card')
            # global drive_alarm
            current_time = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')

            card, count,len = drive_alarm.card_output_fun()
            tip = "上次刷新时间：{}，共{}条显示{}条, 点击查询全部".format(current_time,count,len), 

            return card,tip


        ### 初始化报警创建者
        @app.callback(
                Output("alert_creator", "options"),
                Input('alert_creator', 'id')
            )
        def init_creator_list(id):
            # global creator

            # print('creator',creator)

            ans = []
            for x in creator_list:
                ans.append({
                    "label": x,
                    "value": x
                })
            return ans



        ### 更新报警服务的下拉框
        @app.callback(
                [Output("alert_tbl_ServiceName", "options"), Output("memory_service_info", "data")],
                Input('alert_creator', 'value'),
                
            )
        def init_service_list(creator):
            """
            初始化服务列表
            """
            # print('*************')
            # global SERVER_HOST

            print('creator check',creator)

            if creator is not None:

                res = user_list_api(SERVER_HOST,creator)
                records = res['data']['records']

                ans, cache = [], {}
                for x in records:
                    ans.append({
                        "label": x["service_name"],
                        "value": x["service_name"]
                    })
                    cache[x["id"]] = x["service_name"]
                return ans, cache
            else:
                return no_update, no_update

        ##### 根据报警配置的信息更新drive_alarm的初始化参数
        @app.callback(
            Output('confirm', 'n_clicks'), 
            Input('confirm', 'n_clicks'),
            State('alert_tbl_ServiceName', 'value'),
            State('alert_creator', 'value'),
            State('filter_hours', 'value'),
            State('update_cycle', 'value'),
            State('panel_show_number', 'value'),
            State('panel_image_width', 'value'),
            State('panel_max_height'  , 'value'),
            prevent_initial_call=True
        )
        def update_config(n1,servicename,creator,time_delta,cycle,panel_number,panel_image_width,panel_max_height):
            print('servicename',servicename)
            # print('time_delta',time_delta)
            # print('cycle',cycle)
            # global drive_alarm

            # print('creator new',creator)
            print('service_name old',drive_alarm.servicename)

            if creator:
                drive_alarm.creator = creator
            if servicename:
                drive_alarm.servicename = servicename
            if time_delta:
                drive_alarm.time_delta = time_delta
            if cycle:
                drive_alarm.time_cycle = cycle
            if panel_number:
                drive_alarm.alarm_number = int(panel_number)
            if panel_image_width:
                drive_alarm.panel_image_width = int(panel_image_width)
            if panel_max_height:
                drive_alarm.panel_max_height = int(panel_max_height)
            
            return no_update


        ###报警配置框的确认按钮关闭功能定义
        @app.callback(
            Output("modal_config", "is_open"),
            [Input("alarm_config", "n_clicks"), Input('confirm', 'n_clicks'),],
            [State("modal_config", "is_open")],
        )
        def toggle_modal(n1,n2,is_open):
            if n1 or n2:
                return not is_open
            return is_open
    except:
        try:
            # app.config.external_stylesheets.append('/static/style.css')

            # @app.server.route('/static/<resource>')
            # def serve_static_2(resource):
            #     print('resource',resource)
            #     STATIC_PATH = os.path.join(os.getcwd(), 'demo_project\static')

            #     print('STATIC_PATH',STATIC_PATH)
            #     return flask.send_from_directory(STATIC_PATH, resource)

            @app.callback(
                Output('card_output', 'children'),
                Output("tips_button", "children"),
                Input('apply', 'n_clicks'),
                Input('interval_1m', 'n_intervals')
                # prevent_initial_call=True
            )
            def card_output_fun_2(n1,intervals):
                """
                更新card
                """
                # print('更新card')
                # global drive_alarm
                current_time = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')

                card, count,len = drive_alarm.card_output_fun()
                tip = "上次刷新时间：{}，共{}条显示{}条, 点击查询全部".format(current_time,count,len), 

                return card,tip


            ### 初始化报警创建者
            @app.callback(
                    Output("alert_creator", "options"),
                    Input('alert_creator', 'id')
                )
            def init_creator_list_2(id):
                # global creator

                # print('creator',creator)

                ans = []
                for x in creator_list:
                    ans.append({
                        "label": x,
                        "value": x
                    })
                return ans



            ### 更新报警服务的下拉框
            @app.callback(
                    [Output("alert_tbl_ServiceName", "options"), Output("memory_service_info", "data")],
                    Input('alert_creator', 'value'),
                    
                )
            def init_service_list_2(creator):
                """
                初始化服务列表
                """
                # print('*************')
                # global SERVER_HOST

                print('creator check',creator)

                if creator is not None:

                    res = user_list_api(SERVER_HOST,creator)
                    records = res['data']['records']

                    ans, cache = [], {}
                    for x in records:
                        ans.append({
                            "label": x["service_name"],
                            "value": x["service_name"]
                        })
                        cache[x["id"]] = x["service_name"]
                    return ans, cache
                else:
                    return no_update, no_update

            ##### 根据报警配置的信息更新drive_alarm的初始化参数
            @app.callback(
                Output('confirm', 'n_clicks'), 
                Input('confirm', 'n_clicks'),
                State('alert_tbl_ServiceName', 'value'),
                State('alert_creator', 'value'),
                State('filter_hours', 'value'),
                State('update_cycle', 'value'),
                State('panel_show_number', 'value'),
                State('panel_image_width', 'value'),
                State('panel_max_height'  , 'value'),
                prevent_initial_call=True
            )
            def update_config_2(n1,servicename,creator,time_delta,cycle,panel_number,panel_image_width,panel_max_height):
                print('servicename',servicename)
                # print('time_delta',time_delta)
                # print('cycle',cycle)
                # global drive_alarm

                # print('creator new',creator)
                print('service_name old',drive_alarm.servicename)

                if creator:
                    drive_alarm.creator = creator
                if servicename:
                    drive_alarm.servicename = servicename
                if time_delta:
                    drive_alarm.time_delta = time_delta
                if cycle:
                    drive_alarm.time_cycle = cycle
                if panel_number:
                    drive_alarm.alarm_number = int(panel_number)
                if panel_image_width:
                    drive_alarm.panel_image_width = int(panel_image_width)
                if panel_max_height:
                    drive_alarm.panel_max_height = int(panel_max_height)
                
                return no_update


            ###报警配置框的确认按钮关闭功能定义
            @app.callback(
                Output("modal_config", "is_open"),
                [Input("alarm_config", "n_clicks"), Input('confirm', 'n_clicks'),],
                [State("modal_config", "is_open")],
            )
            def toggle_modal_2(n1,n2,is_open):
                if n1 or n2:
                    return not is_open
                return is_open
        except:
            app.config.external_stylesheets.append('/static/style.css')

            @app.server.route('/static/<resource>')
            def serve_static_3(resource):
                print('resource',resource)
                STATIC_PATH = os.path.join(os.getcwd(), 'demo_project\static')

                print('STATIC_PATH',STATIC_PATH)
                return flask.send_from_directory(STATIC_PATH, resource)

            @app.callback(
                Output('card_output', 'children'),
                Output("tips_button", "children"),
                Input('apply', 'n_clicks'),
                Input('interval_1m', 'n_intervals')
                # prevent_initial_call=True
            )
            def card_output_fun_3(n1,intervals):
                """
                更新card
                """
                # print('更新card')
                # global drive_alarm
                current_time = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')

                card, count,len = drive_alarm.card_output_fun()
                tip = "上次刷新时间：{}，共{}条显示{}条, 点击查询全部".format(current_time,count,len), 

                return card,tip


            ### 初始化报警创建者
            @app.callback(
                    Output("alert_creator", "options"),
                    Input('alert_creator', 'id')
                )
            def init_creator_list_3(id):
                # global creator

                # print('creator',creator)

                ans = []
                for x in creator_list:
                    ans.append({
                        "label": x,
                        "value": x
                    })
                return ans



            ### 更新报警服务的下拉框
            @app.callback(
                    [Output("alert_tbl_ServiceName", "options"), Output("memory_service_info", "data")],
                    Input('alert_creator', 'value'),
                    
                )
            def init_service_list_3(creator):
                """
                初始化服务列表
                """
                # print('*************')
                # global SERVER_HOST

                print('creator check',creator)

                if creator is not None:

                    res = user_list_api(SERVER_HOST,creator)
                    records = res['data']['records']

                    ans, cache = [], {}
                    for x in records:
                        ans.append({
                            "label": x["service_name"],
                            "value": x["service_name"]
                        })
                        cache[x["id"]] = x["service_name"]
                    return ans, cache
                else:
                    return no_update, no_update

            ##### 根据报警配置的信息更新drive_alarm的初始化参数
            @app.callback(
                Output('confirm', 'n_clicks'), 
                Input('confirm', 'n_clicks'),
                State('alert_tbl_ServiceName', 'value'),
                State('alert_creator', 'value'),
                State('filter_hours', 'value'),
                State('update_cycle', 'value'),
                State('panel_show_number', 'value'),
                State('panel_image_width', 'value'),
                State('panel_max_height'  , 'value'),
                prevent_initial_call=True
            )
            def update_config_3(n1,servicename,creator,time_delta,cycle,panel_number,panel_image_width,panel_max_height):
                print('servicename',servicename)
                # print('time_delta',time_delta)
                # print('cycle',cycle)
                # global drive_alarm

                # print('creator new',creator)
                print('service_name old',drive_alarm.servicename)

                if creator:
                    drive_alarm.creator = creator
                if servicename:
                    drive_alarm.servicename = servicename
                if time_delta:
                    drive_alarm.time_delta = time_delta
                if cycle:
                    drive_alarm.time_cycle = cycle
                if panel_number:
                    drive_alarm.alarm_number = int(panel_number)
                if panel_image_width:
                    drive_alarm.panel_image_width = int(panel_image_width)
                if panel_max_height:
                    drive_alarm.panel_max_height = int(panel_max_height)
                
                return no_update


            ###报警配置框的确认按钮关闭功能定义
            @app.callback(
                Output("modal_config", "is_open"),
                [Input("alarm_config", "n_clicks"), Input('confirm', 'n_clicks'),],
                [State("modal_config", "is_open")],
            )
            def toggle_modal_3(n1,n2,is_open):
                if n1 or n2:
                    return not is_open
                return is_open
    #### card 报警列表框
    # print('panel_callback start')
    # app.config.external_stylesheets.append([dbc.themes.BOOTSTRAP,'/static/style.css'])
    # app.config.external_stylesheets.append(['/static/style.css'])
    # app.config.external_stylesheets=['/static/style.css',dbc.themes.BOOTSTRAP]
    # try:
    #     app.config.external_stylesheets.append('/static/style.css')
        
    #     @app.server.route('/static/<resource>')
    #     @authenticate_admin
    #     # def serve_static(resource):
    #     #     return None
    #     # def serve_static(resource):
    #     #     # print('resource',resource)
    #     #     STATIC_PATH = os.path.join(os.getcwd(), 'drive_screen\static')

    #     #     print('STATIC_PATH',STATIC_PATH)
    #     #     return flask.send_from_directory(STATIC_PATH, resource)

    #     ### 更新周期变更
    #     @app.callback(
    #         Output('interval_1m', 'interval'),
    #         Input('confirm', 'n_clicks'),
    #         State('update_cycle', 'value'),
    #     )
    #     @update_cycle_fun
    #     # def update_cycle_fun(n1,n):
    #     #     # print('interval_new',1000*int(n))
    #     #     return 1000*int(n)

    #     @app.callback(
    #         Output('card_output', 'children'),
    #         Output("tips_button", "children"),
    #         Input('apply', 'n_clicks'),
    #         Input('interval_1m', 'n_intervals'),
            
    #         # prevent_initial_call=True
    #     )
    #     @card_output_deco(drive_alarm)
    #     # def card_output_fun(n1,intervals,drive_alarm):
    #     #     """
    #     #     更新card
    #     #     """
    #     #     # print('更新card')
    #     #     # global drive_alarm
    #     #     current_time = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')

    #     #     card, count,len = drive_alarm.card_output_fun()
    #     #     tip = "上次刷新时间：{}，共{}条显示{}条, 点击查询全部".format(current_time,count,len), 

    #     #     return card,tip


    #     ### 初始化报警创建者
    #     @app.callback(
    #             Output("alert_creator", "options"),
    #             Input('alert_creator', 'id')
    #         )
    #     @init_creator_list_deco(creator_list)
    #     # def init_creator_list(id):
    #     #     # global creator

    #     #     # print('creator',creator)

    #     #     ans = []
    #     #     for x in creator_list:
    #     #         ans.append({
    #     #             "label": x,
    #     #             "value": x
    #     #         })
    #     #     return ans



    #     ### 更新报警服务的下拉框
    #     @app.callback(
    #             [Output("alert_tbl_ServiceName", "options"), Output("memory_service_info", "data")],
    #             Input('alert_creator', 'value'),
                
    #         )
    #     @init_service_list_deco(SERVER_HOST)
    #     # def init_service_list(creator):
    #     #     """
    #     #     初始化服务列表
    #     #     """
    #     #     # print('*************')
    #     #     # global SERVER_HOST

    #     #     # print('creator check',creator)

    #     #     if creator is not None:

    #     #         res = user_list_api(SERVER_HOST,creator)
    #     #         records = res['data']['records']

    #     #         ans, cache = [], {}
    #     #         for x in records:
    #     #             ans.append({
    #     #                 "label": x["service_name"],
    #     #                 "value": x["service_name"]
    #     #             })
    #     #             cache[x["id"]] = x["service_name"]
    #     #         return ans, cache
    #     #     else:
    #     #         return no_update, no_update


    #     ###报警配置框的确认按钮关闭功能定义
    #     @app.callback(
    #         Output("modal_config", "is_open"),
    #         [Input("alarm_config", "n_clicks"), Input('confirm', 'n_clicks'),],
    #         [State("modal_config", "is_open")],
    #     )
    #     @toggle_modal_fun
    #     def compile_1():
    #         pass    
    # except:
    #     app.config.external_stylesheets.append('/static/style.css')
        
    #     @app.server.route('/static/<resource>')
    #     @authenticate_admin
    #     # def serve_static(resource):
    #     #     return None
    #     # def serve_static(resource):
    #     #     # print('resource',resource)
    #     #     STATIC_PATH = os.path.join(os.getcwd(), 'drive_screen\static')

    #     #     print('STATIC_PATH',STATIC_PATH)
    #     #     return flask.send_from_directory(STATIC_PATH, resource)

    #     ### 更新周期变更
    #     @app.callback(
    #         Output('interval_1m', 'interval'),
    #         Input('confirm', 'n_clicks'),
    #         State('update_cycle', 'value'),
    #     )
    #     @update_cycle_fun
    #     # def update_cycle_fun(n1,n):
    #     #     # print('interval_new',1000*int(n))
    #     #     return 1000*int(n)

    #     @app.callback(
    #         Output('card_output', 'children'),
    #         Output("tips_button", "children"),
    #         Input('apply', 'n_clicks'),
    #         Input('interval_1m', 'n_intervals'),
            
    #         # prevent_initial_call=True
    #     )
    #     @card_output_deco(drive_alarm)
    #     # def card_output_fun(n1,intervals,drive_alarm):
    #     #     """
    #     #     更新card
    #     #     """
    #     #     # print('更新card')
    #     #     # global drive_alarm
    #     #     current_time = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')

    #     #     card, count,len = drive_alarm.card_output_fun()
    #     #     tip = "上次刷新时间：{}，共{}条显示{}条, 点击查询全部".format(current_time,count,len), 

    #     #     return card,tip


    #     ### 初始化报警创建者
    #     @app.callback(
    #             Output("alert_creator", "options"),
    #             Input('alert_creator', 'id')
    #         )
    #     @init_creator_list_deco(creator_list)
    #     # def init_creator_list(id):
    #     #     # global creator

    #     #     # print('creator',creator)

    #     #     ans = []
    #     #     for x in creator_list:
    #     #         ans.append({
    #     #             "label": x,
    #     #             "value": x
    #     #         })
    #     #     return ans



    #     ### 更新报警服务的下拉框
    #     @app.callback(
    #             [Output("alert_tbl_ServiceName", "options"), Output("memory_service_info", "data")],
    #             Input('alert_creator', 'value'),
                
    #         )
    #     @init_service_list_deco(SERVER_HOST)
    #     # def init_service_list(creator):
    #     #     """
    #     #     初始化服务列表
    #     #     """
    #     #     # print('*************')
    #     #     # global SERVER_HOST

    #     #     # print('creator check',creator)

    #     #     if creator is not None:

    #     #         res = user_list_api(SERVER_HOST,creator)
    #     #         records = res['data']['records']

    #     #         ans, cache = [], {}
    #     #         for x in records:
    #     #             ans.append({
    #     #                 "label": x["service_name"],
    #     #                 "value": x["service_name"]
    #     #             })
    #     #             cache[x["id"]] = x["service_name"]
    #     #         return ans, cache
    #     #     else:
    #     #         return no_update, no_update


    #     ###报警配置框的确认按钮关闭功能定义
    #     @app.callback(
    #         Output("modal_config", "is_open"),
    #         [Input("alarm_config", "n_clicks"), Input('confirm', 'n_clicks'),],
    #         [State("modal_config", "is_open")],
    #     )
    #     @toggle_modal_fun
    #     def compile_2():
    #         pass

    # dynamic_function_loader.load("def foo{i}():\n    return 'bar'")

    # @app.callback(
    #     Output("modal_config", "id"),
    #     [Input("alarm_config", "n_clicks"), Input('confirm', 'n_clicks'),],
    #     [State("modal_config", "is_open")],
    # )
    # try:
    #     if i ==1:
    #         return dynamic_function_loader.load("def foo{i}():\n    return 'bar'")
    # except Exception as e:
    #     print(e)
    #     return no_update
    # exec(dynamic_function_loader.load("def foo{i}():\n    return 'bar'"))

    # @retry()
    # def foo():
    #     pass
    # compile('def gfg(): return "GEEKSFORGEEKS"', "<string>", "exec")
    # @exception_handler
    # def toggle_modal_1(n):
    
    #     return no_update
   


import dash_bootstrap_components as dbc
import dash_html_components as html
import dash_core_components as dcc

from dash import Input, Output,State, callback


modal_config = [
           dbc.Modal(
                [
                    dbc.ModalHeader(dbc.ModalTitle("报警配置")),
                    dbc.ModalBody(
                        [
                            html.P("请选择报警创建者"),
                            dcc.Dropdown(
                                id='alert_creator',
                                clearable=True,
                                className="dcc_control",
                                multi=False,
                            ),
                            html.P("请选择报警服务名"),
                            dcc.Dropdown(
                                id='alert_tbl_ServiceName',
                                # value='dt-casting-analysis',
                                # options=charts_biz.get_service_list(),
                                clearable=True,
                                className="dcc_control",
                                multi=False,
                            ),
                            html.Br(),
                            html.P("请输入显示最近多少小时的未处理报警",style={'padding': '20px 0px 0px 0px'},),
                            dcc.Input(id="filter_hours", type="text", placeholder="", style={'marginRight':'10px'}, value = '1'),
                           html.P("请输入每隔多少秒刷新一次(最小设置10s)" ,style={'padding': '20px 0px 0px 0px'}),
                            dcc.Input(id="update_cycle", type="text", placeholder="", style={'marginRight':'10px'}, value = '10'),
                            html.P("请输入每次最多显示多少条报警",style={'padding': '20px 0px 0px 0px'}),
                            dcc.Input(id="panel_show_number", type="text", placeholder="", style={'marginRight':'10px'}, value = '4'),
                            html.P("如果有图片，请输入图片的宽度",style={'padding': '20px 0px 0px 0px'}),
                            dcc.Input(id="panel_image_width", type="text", placeholder="", style={'marginRight':'10px'}, value = '300'),
                            html.P("报警卡片最大高度设置",style={'padding': '20px 0px 0px 0px'}),
                            dcc.Input(id="panel_max_height", type="text", placeholder="", style={'marginRight':'10px'}, value = '1000'),
                        ]
                    ),
                    dbc.ModalFooter(
                        dbc.Button(
                            "确认", id="confirm", className="ms-auto", n_clicks=0
                        )
                    ),
                ],
                # fullscreen = ['sm-down', 'md-down', 'lg-down', 'xl-down', 'xxl-down'],
                fullscreen = 'xxl-down',
                id="modal_config",
                is_open=False,
                centered=True
            ),
        ]

def alarm_panel_layout_fun(Front_Host, width = 5, height = 800):
    '''
    width: int, the width of the panel (1-12)
    height: int, the height of the panel in unit px
    
    '''
    if width>=4:

        card = dbc.Col(
                    dbc.Container([
                        dcc.Interval(id='interval_1m', interval = 60*1000, n_intervals=0),# ms
                        dcc.Store(id='memory_service_info'),
                        html.Div(children= modal_config),
                        dbc.Button('刷新',id= 'apply'),
                        dbc.Button('配置',id= 'alarm_config',style={"margin-left": "5px"}),
                        dbc.Button(id = 'tips_button',href=Front_Host+'/alert/list/', outline=True, color="success", className="me-1",style={"margin-left": "5px"}),
                        html.Br(),
                        dbc.Toast(
                            dbc.Container([
                                
                                dbc.Container(
                                    id="card_output", 
                                    style={'padding': '0px 0px 0px 0px'},
                                    ),
                                ],
                                id="my_card", 
                                style={'padding': '0px 0px 0px 0px'},
                            ),
                            style={'overflow-y': 'scroll',"maxWidth": "2000px", 'width': 159*width,"height":height,'padding': '0px 0px 0px 0px'},
                        ),
                    ]
                    ),
                    width=width,
                    style={'padding': '0px 0px 0px 0px'},

                )
    else:
         card = dbc.Col(
                    dbc.Container([
                        dcc.Interval(id='interval_1m', interval = 60*1000, n_intervals=0),# ms
                        dcc.Store(id='memory_service_info'),
                        html.Div(children= modal_config),
                        # dbc.Row([
                        #     # dbc.Col(
                        #     #     dbc.Button('刷新',id= 'apply'),
                        #     #     width = 4
                        #     # ),
                        #     # dbc.Col(
                        #     #     dbc.Button('配置',id= 'alarm_config'),
                        #     #     width = 4
                        #     # ),
                        #     dbc.Button('刷新',id= 'apply'),
                        #     dbc.Button('配置',id= 'alarm_config',style={"margin-left": "5px"}),
                        #     ]
                        # ),
                        dbc.Button('刷新',id= 'apply'),
                        dbc.Button('配置',id= 'alarm_config',style={"margin-left": "5px"}),
                        dbc.Row(
                            dbc.Col(
                                dbc.Button(id = 'tips_button',href=Front_Host+'/alert/list/', outline=True, color="success", className="me-1"),
                            ),
                            style={"padding": "5px 0px 0px 0px"}
                        ),
                        dbc.Toast(
                            dbc.Container([
                                
                                dbc.Container(
                                    id="card_output",
                                    style={'padding': '0px 0px 0px 0px'},
                                    ),
                                ],
                                id="my_card",
                                style={'padding': '0px 0px 0px 0px'},
                            ),
                            # style={'overflow-y': 'scroll','overflow-x': 'hidden',"maxWidth": "2000px", 'width': 160*width,"height":height},
                            # style={"maxWidth": "2000px", 'width': 160*width,"height":height},
                            style={'overflow': 'scroll',"maxWidth": "2000px", 'width': 159*width,"height":height,'padding': '0px 0px 0px 0px'},

                        ),]
                    ),
                    width=width,
                )
        
    return card



button_style_full = {
    "display": "inline-block",
    "height": 40,
    "width": 90,
    "padding": "0 10 10 10",
    "color": "#555",
    "textAlign": "center",
    "fontSize": 11,
    "fontWeight": 600,
    # "lineHeight": 38,
    "letterSpacing": .1,
    "textTransform": "uppercase",
    "textDecoration": "none",
    "whiteSpace": "nowrap",
    "backgroundColor": "transparent",
    "borderRadius": 4,
    "border": "1 solid #bbb",
    "cursor": "pointer",
    "boxSizing": "border-box"
}

modal_info_style= {
    # 'background-color': '#D1FFFD',
    # "color": "#0057D8",
    'background-color': '#FFFFFF',
    "color": "#030303",
}


# content_style_home = {
#     "padding-top": "0px",
#     "padding-left": "0px",
#     "padding-right": "0px",
#     "padding-bottom": "0px",
#     "margin-top": "0px",
#     "margin-left": "0px",
#     "margin-right": "0px",
#     "margin-bottom": "0px",
# }


# font_style = {
#    'background-color': '#c9170a',
#     'color': '#fff',
#     'text-align': 'center',
#     'height': '88px',
#     'font-size': '14px',
#     'font-weight': '400',
#     'line-height': '88px',
#     'letter-spacing': '-.25px',
# }



import requests


def alarm_list_api(SERVER_HOST,filter,order,page,size):

    res = requests.get(f"{SERVER_HOST}/alarm/list", json={"filter": filter,'order':order,'page':page, 'size':size}, verify=False).json()

    return res



def user_list_api(SERVER_HOST,creator):

    res = requests.get(f"{SERVER_HOST}/user/list", json={'creator' : creator},verify=False).json()

    return res



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['drive_screen']

package_data = \
{'': ['*'], 'drive_screen': ['static/*']}

install_requires = \
['Flask>=2.0.1,<3.0.0',
 'Pillow>=9.2.0,<10.0.0',
 'PyMySQL>=1.0.2,<2.0.0',
 'dash-core-components>=2.0.0,<3.0.0',
 'dash-html-components>=2.0.0,<3.0.0',
 'dash>=2.0.0,<3.0.0',
 'dash_bootstrap_components>=1.0.1rc1,<2.0.0',
 'pandas>=1.1.5,<2.0.0',
 'plotly>=5.2.1,<6.0.0',
 'requests>=2.27.1,<3.0.0']

setup_kwargs = {
    'name': 'drive-screen',
    'version': '0.1.9',
    'description': 'add multi page panel function support & fix bar color',
    'long_description': '',
    'author': 'shengqin ding',
    'author_email': 'dshengq@163.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)




from functools import wraps
from drive_screen.alarm_api import user_list_api
from dash import no_update

import datetime
import flask
import os

def authenticate_admin(func):
    @wraps(func)
    def authenticate_and_call(*args, **kwargs):
        STATIC_PATH = os.path.join(os.getcwd(), 'drive_screen\static')

        print('STATIC_PATH',STATIC_PATH)
        return flask.send_from_directory(STATIC_PATH, 'style.css')

    return authenticate_and_call


def update_cycle_fun(func):
    @wraps(func)
    def update_cycle(*args, **kwargs):

        return  1000*int(args[1])
    return update_cycle


def card_output_deco(drive_alarm):
    def card_output_fun(func):
        @wraps(func)
        def card_output(*args):

            """
            更新card
            """
            current_time = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')

            card, count,length = drive_alarm.card_output_fun()
            tip = "上次刷新时间：{}，共{}条显示{}条, 点击查询全部".format(current_time,count,length), 

            return card,tip
        return card_output
    return card_output_fun


def init_creator_list_deco(creator_list):
    def init_creator_list_fun(func):
        @wraps(func)
        def init_creator_list(*args, **kwargs):
            ans = []
            for x in creator_list:
                ans.append({
                    "label": x,
                    "value": x
                })
            return ans
        return init_creator_list
    return init_creator_list_fun
    

def init_service_list_deco(SERVER_HOST):
    def init_service_list_func(func):
        @wraps(func)
        def init_service_list(*args, **kwargs):

            """
            初始化服务列表
            """
            # print('*************')
            # global SERVER_HOST

            # print('creator check',creator)

            if args[0] is not None:
                res = user_list_api(SERVER_HOST,args[0])
                records = res['data']['records']

                ans, cache = [], {}
                for x in records:
                    ans.append({
                        "label": x["service_name"],
                        "value": x["service_name"]
                    })
                    cache[x["id"]] = x["service_name"]
                return ans, cache
            else:
                return no_update, no_update
        return init_service_list
    return init_service_list_func



def update_config_deco(drive_alarm):
    def update_config_fun(func):
        @wraps(func)
        def update_config(*args, **kwargs):
            if args[2]:
                drive_alarm.creator = args[2]
            if args[1]:
                drive_alarm.servicename = args[2]
            if args[3]:
                drive_alarm.time_delta = args[3]
            if args[4]:
                drive_alarm.time_cycle = args[4]
            if args[5]:
                drive_alarm.alarm_number = int(args[5])
            if args[6]:
                drive_alarm.panel_image_width = int(args[6])
            if args[7]:
                drive_alarm.panel_max_height = int(args[7])
        
        return no_update
    return update_config_fun


def toggle_modal_fun(func):
    @wraps(func)
    def toggle_modal(*args, **kwargs):

        if args[0] or args[1]:
            return not args[2]
        return args[2]        
    return toggle_modal






from itertools import cycle
import pandas as pd
import base64
import dash_bootstrap_components as dbc
import dash_html_components as html
import json
from dash.development.base_component import Component
from dash.dash import no_update
from PIL import Image, ImageDraw as D
from dash import dash_table,dcc
from io import BytesIO as _BytesIO
from drive_screen.alarm_api import *
from drive_screen.style import button_style_full,modal_info_style
import datetime


button_cn_map = {'valid_alarm_button':'有效报警','invalid_alarm_button':'无效报警','respond_button':'添加报警评论'}

class  DriveAlarmPanel(object):
    '''
    __init__ : 初始化参数

    method：
        modal_output_fun： 返回警弹出框
        card_output_fun: 返回报警卡片，符合筛选条件的报警个数，报警总个数
    '''

    def __init__(self,servicename,creator,time_delta = 1,time_cycle =10,SERVER_HOST=None,alarm_number =4,panel_image_width = 500,window_image_width = 400,panel_max_height = 1000, window_size = 'lg'):
        '''
        servicename: 告警服务名称
        creator : 创建者
        SERVER_HOST： api address
        time_delta: int,  unit:hour    recent hours you want to search the alarm
        time_cycle: int,  unit:minute  the update cycle
        panel_number: int, the number of panels you want to show
        panel_image_width": int,  unit:pixel  the width of the image in the panel 
        window_image_width: int,  unit:pixel  the width of the image in the window 
        panel_max_height: int,  unit:pixel  the maximum height of the panel 
        window_size: sm, lg, xl for small, large or extra large sized modals, or leave undefined for default size.
        
        '''
        # print('初始化类')
        self.servicename = servicename
        self.time_delta = time_delta
        self.time_cycle = time_cycle
        self.creator = creator
        self.SERVER_HOST = SERVER_HOST
        self.alarm_number = alarm_number
        self.panel_image_width = panel_image_width
        self.window_image_width = window_image_width
        self.panel_max_height = panel_max_height
        self.window_size = window_size

        if self.SERVER_HOST is None:
            print('SERVER_HOST is not defined')


    def b64_to_pil(self,image_base64, width = 0):
            
            image_base64 = image_base64.replace('data:image/.jpeg;base64,','')
            image_base64 = image_base64.replace('data:image/.png;base64,','')
            image_base64 = image_base64.replace('data:image/png;base64,','')
        #    data:image/png;base64,

            
    #         print(image_base64)
            decoded = base64.b64decode(image_base64)
            buffer = _BytesIO(decoded)
            im = Image.open(buffer)
            # if width  :
            origin_width, origin_height = im.size
            new_height = int(origin_height*width/origin_width)
            newsize = (width, new_height)
            im = im.resize(newsize)

            return im


    def display_modal2(self,i,data, feedback_key,width):
        print('开始 display_modal2')

        ans = []
        # show image
        if data.get("content").get("image_base64"):
            for index, img in enumerate(data.get("content").get("image_base64")):
                if 'data:image/' in img:
                    img_base64_encode = img.split(',')[1]
                else:
                    img_base64_encode = img

                    # print('image_base64_error',data.get("content").get("image_base64"))
                ans.append(
                    # dcc.Graph(id=f'annotation_{str(index)}', figure=b64_to_pil(img_base64_encode)),
                    html.Img(src=self.b64_to_pil(img_base64_encode,width=width)),
                )

        # text
        title, text = "", ""
        if data.get("content").get("alarm_title"):
            title += data.get("content").get("alarm_title") + "  "
        if data.get("content").get("message_title"):
            title += data.get("content").get("message_title")

        if data.get("content").get("message_text"):
            text += data.get("content").get("message_text")
        if title:
            ans.append(html.H5(title))
        if text:
            ans.append(html.P(text))

        # table
        if data.get("content").get("table"):
            ans.append(
                dbc.Table.from_dataframe(pd.DataFrame(json.loads(data.get("content").get("table"))), striped=True,
                                        bordered=True, hover=True)
            )

        # ans.extend(
        #     [
        #         html.Br(),
        #         html.Hr(className="my-2")
        #     ]
        # )
        # button
        buttons = []
        if data.get("content").get("buttons"):
            for obj in data.get("content").get("buttons"):
                buttons.append(
                    dbc.Button(obj.get("title"), href=obj.get("url"), outline=True, className="me-1"),
                    # dbc.Button("无效报警", outline=True, color="warning", className="me-1"),
                    # dbc.Button("添加评论", outline=True, color="secondary", className="me-1"),
                )
        additional_functions_mapping = {
            'valid_alarm_button': "/alarm/respond/valid/",
            'invalid_alarm_button': "/alarm/respond/invalid/",
            'respond_button': "/alarm/comment/",
        }
        if data.get("additional_functions"):
            for name in data.get("additional_functions"):
                buttons.append(
                    dbc.Button(
                        button_cn_map[name],
                        href=self.SERVER_HOST + additional_functions_mapping.get(name) + feedback_key,
                        outline=True, color="secondary", className="me-1"
                    )
                )
        button = [html.P(buttons, className="lead")]
        
        boder_nr = 'default'
        if 'severity' in data:
            severity = data.get("severity")
            if severity <= 20:
                boder_nr = 'green01'
            elif severity <= 40:
                boder_nr = 'green02'
            elif severity <= 60:
                boder_nr = 'yellow'
            elif severity <= 80:
                boder_nr = 'orange'
            elif severity <= 100:
                boder_nr = 'red'


        modal = dbc.Modal(
                    html.Div([
                        dbc.ModalHeader(
                            style = modal_info_style,
                            children = dbc.Label('alarm', style={'text-align': 'left', 'font-size': '150%','font-style': 'italic','font-weight':'bold','font-color': '#0057D8', 'text-transform': 'lowercase'}),
                            close_button = True,
                            class_name = "close-button"
                        ),
                        dbc.ModalBody(
                            children = ans
                        ),
                        dbc.ModalFooter(
                            style = modal_info_style,
                            children = button  
                        ), 
                    ],
                    
                    className='divBorder_'+boder_nr,
                ),
                backdrop = 'static',
                is_open=True,
                centered=True,
                size=self.window_size,
                fullscreen = 'sm-down', #'md-down', 'lg-down', 'xl-down', 'xxl-down',
                scrollable=True,
                style={"max-width": "none", "width": "90%"},
            )

        print('结束 display_modal2')
        

        return modal



    def display_modal1(self, i, data,feedback_key):
        print('开始 display_modal1')
        ans = []
        # text
        title, text = "", ""
        if data.get("content").get("title"):
            title += data.get("content").get("title") + "  "
        if data.get("content").get("sub_title"):
            title += data.get("content").get("sub_title")

        if data.get("content").get("message_text"):
            text += data.get("content").get("message_text")
        if title:
            ans.append(html.H5(title))
        if text:
            ans.append(html.P(text))

        # table
        if data.get("content").get("table"):
            ans.append(
                dbc.Table.from_dataframe(pd.DataFrame(json.loads(data.get("content").get("table"))), striped=True,
                                        bordered=True, hover=True)
            )

        # print('ans1',ans)
        buttons = []
        if data.get("content").get("buttons"):
            for obj in data.get("content").get("buttons"):
                buttons.append(
                    dbc.Button(obj.get("title"), href=obj.get("url"), outline=True,  className="me-1"),
                    # dbc.Button("无效报警", outline=True, color="warning", className="me-1"),
                    # dbc.Button("添加评论", outline=True, color="secondary", className="me-1"),
                )
        additional_functions_mapping = {
            'valid_alarm_button': "/alarm/respond/valid/",
            'invalid_alarm_button': "/alarm/respond/invalid/",
            'respond_button': "/alarm/comment/",
        }
        if data.get("additional_functions"):
            for name in data.get("additional_functions"):
                buttons.append(
                    dbc.Button(
                        button_cn_map[name],
                        href=self.SERVER_HOST + additional_functions_mapping.get(name) + feedback_key,
                        outline=True, color="secondary", className="me-1"
                    )
                )
        button = [html.P(buttons, className="lead")]


        # print('ans2',ans)
        boder_nr = 'default'
        if 'severity' in data:
            severity = data.get("severity")
            if severity <= 20:
                boder_nr = 'green01'
            elif severity <= 40:
                boder_nr = 'green02'
            elif severity <= 60:
                boder_nr = 'yellow'
            elif severity <= 80:
                boder_nr = 'orange'
            elif severity <= 100:
                boder_nr = 'red'

        modal = dbc.Modal(
                    html.Div([
                        dbc.ModalHeader(
                            style = modal_info_style,
                            children = dbc.Label('alarm', style={'text-align': 'left', 'font-size': '150%','font-style': 'italic','font-weight':'bold','font-color': '#0057D8', 'text-transform': 'lowercase'}),
                            close_button = True,
                            class_name = "close-button"
                        ),
                        dbc.ModalBody(
                            children = ans,
                            style={"overflow": "scroll"},
                        ),
                        dbc.ModalFooter(
                            style = modal_info_style,
                            children = button  
                        ), 
                        ],
                        className='divBorder_'+boder_nr
                    ),
                    backdrop = 'static',
                    is_open=True,
                    # centered=True,
                    size=self.window_size,
                    # fullscreen = 'sm-down', #'md-down', 'lg-down', 'xl-down', 'xxl-down',
                    # scrollable=True,
                    style={"max-width": "none", "width": "90%"},
                    id = f'modal_{i}',
                )
                
        print('结束 display_modal1')
        
        return modal



    def modal_output_fun(self,interval,children):
        print('开始modaloutput')

        start_time = '2022-07-06 15:01:06'
        end_time = '2022-07-06 16:00:06'

        # start_time = datetime.datetime.strftime(datetime.datetime.now() - datetime.timedelta(seconds=int(interval/1000)+5) ,'%Y-%m-%d %H:%M:%S')
        # end_time = datetime.datetime.strftime(datetime.datetime.now() ,'%Y-%m-%d %H:%M:%S')

        print('start_time',start_time)
        print('end_time',end_time)
        print('servicename',self.servicename)

        filter1 = {'username': self.servicename, 'start_time': start_time, 'end_time': end_time, 'read':True, 'status':''}
        order = {'field':'create_time', 'direction':'desc'}

        print(filter1)
    
        # res = requests.get(f"{SERVER_HOST}/alarm/list", json={"filter": filter}, verify=False).json()
        # res = requests.get(f"{SERVER_HOST}/alarm/list", json={"filter": filter1,'order':order,'page':1, 'size':4}, verify=False).json()
        res = alarm_list_api(self.SERVER_HOST,filter1,order,1,self.alarm_number)

        # print(res)

        if res['data']['records']==[]:

            print('no_update')

            return no_update

        else:
            print('has_update')

            alarm_data = res['data']['records']

            modal_list = []


            for i in alarm_data:
                data = i["content"]
                feedback_key = i["feedback_key"]
                template = i["content"]["template"]
                # print(template)

                if template in ('message+image_v2', 'message+image_v1'):
                    modal_list.append(self.display_modal2(i, data,feedback_key,width=self.window_image_width))
                elif template == "message_only_v1":
                    modal_list.append(self.display_modal1(i, data,feedback_key))
            

            # print('modal_list',modal_list)
            if children is None:
                children = []
            children.extend(modal_list)
        
            
            return children 

    def screen_msgiv1(self,data,create_time, feedback_key):

        ans_table,ans_title,ans_text,ans_button = [],[],[],[]

        # text
        title, text = "", ""
        if data.get("content").get("title"):
            title += data.get("content").get("title")
        if data.get("content").get("sub_title"):
            text += data.get("content").get("sub_title")

        if title:
            ans_title.append(html.H5(title))
        if text:
            ans_text.append(html.P(text))
        
        # print('ans_title',ans_title)
        # print('ans_text',ans_text)


        # table
        if data.get("content").get("table"):
            df_table = pd.DataFrame(json.loads(data.get("content").get("table")))
            if df_table.shape[0]>3:
                df_table = df_table.iloc[0:3,:]
            ans_table.append(      
                dbc.Table.from_dataframe(
                    df_table, striped=True, bordered=True, hover=True,size = 'sm',responsive =True,
                    style={'padding': '0px 0px 0px 0px'},
                )
            )

        if data.get("content").get("buttons"):
            for obj in data.get("content").get("buttons"):
                ans_button.append(
                    dbc.Button(obj.get("title"), href=obj.get("url"), outline=True, color="secondary",  className="me-1"),
                    # dbc.Button('无效报警', href=obj.get("url"), outline=True, color="success", className="me-1"),

                    # dbc.Button("无效报警", outline=True, color="warning", className="me-1"),
                    # dbc.Button("添加评论", outline=True, color="secondary", className="me-1"),
                )
        additional_functions_mapping = {
            'valid_alarm_button': "/alarm/respond/valid/",
            'invalid_alarm_button': "/alarm/respond/invalid/",
            'respond_button': "/alarm/comment/",
        }
        if data.get("additional_functions"):
            for name in data.get("additional_functions"):
                ans_button.append(
                    dbc.Button(
                        button_cn_map[name],
                        href=self.SERVER_HOST + additional_functions_mapping.get(name) + feedback_key,
                        outline=True, color="secondary", className="me-1",style={'padding': '0px 0px 0px 0px'},
                    )
                )

        
        boder_nr = 'default'
        if 'severity' in data:
            severity = data.get("severity")
            if severity <= 20:
                boder_nr = 'green01'
            elif severity <= 40:
                boder_nr = 'green02'
            elif severity <= 60:
                boder_nr = 'yellow'
            elif severity <= 80:
                boder_nr = 'orange'
            elif severity <= 100:
                boder_nr = 'red'

        if ans_table!=[]:
            card = [dbc.Toast(
                        dbc.Row([
                            dbc.Col(
                                ans_table,
                                width=9,
                                style={'padding': '0px 2px 0px 15px'},
                            ),
                            dbc.Col([
                                html.P(
                                    '报警时间：'+create_time,
                                    className="card-text",
                                    style={'padding': '0px 10px 0px 0px'},
                                ),
                                html.P(
                                    ans_title,
                                    className="card-text",
                                    style={'padding': '0px 0px 0px 0px'},
                                ),
                                html.P(
                                    ans_text,
                                    className="card-text",
                                    style={'padding': '0px 0px 0px 0px'},
                                ),
                                dbc.Col(ans_button,
                                    style={"flexWrap":"nowrap",'padding': '0px 0px 0px 0px'}
                                ),
                                ],
                                width=3,
                                style={'padding': '0px 0px 0px 0px'},
                            )
                            ],
                            style={'padding': '0px 0px 0px 0px'},

                        ),
                        className='divBorder_'+boder_nr,
                        # style={'padding': '0px 5px 0px 0px'},
                        style={"maxWidth": "2000px", "width":800,"maxHeight":self.panel_max_height,'padding': '0px 30px 0px 0px'},


                    )
                ]
        else:
            card = [dbc.Toast(
                        dbc.Row([
                            dbc.Col([
                                html.P(
                                    '报警时间：'+create_time,
                                    className="card-text",
                                    style={'padding': '0px 10px 0px 0px'},
                                ),
                                html.P(
                                    ans_title,
                                    className="card-text",
                                    style={'Margin': '0px 0px 0px 0px'},
                                ),
                                html.P(
                                    ans_text,
                                    className="card-text",
                                    style={'Margin': '0px 0px 0px 0px'},
                                ),
                                dbc.Col(ans_button,
                                    style={"flexWrap":"nowrap",'padding': '0px 0px 0px 0px'}
                                ),
                                ],
                                style={'padding': '0px 0px 0px 0px'},
                            ),
                            ],
                            style={'padding': '0px 0px 0px 0px'},
                        ),
                        className='divBorder_'+boder_nr,
                        # style={'padding': '0px 5px 0px 0px'},
                        style={"maxWidth": "2000px", "width":800,"maxHeight":self.panel_max_height,'padding': '0px 30px 0px 0px'},
                    )
                ]

        card.extend([html.Div(dcc.Markdown('''---''',),style={'padding': '0px 0px 0px 0px'})])
            
        return  card



    def screen_msgiv2(self,data, create_time,feedback_key,width):

        ans_image_table,ans_title,ans_text,ans_button = [],[],[],[]
        # show image
        if data.get("content").get("image_base64"):
            for index, img in enumerate(data.get("content").get("image_base64")):
                if 'data:image/' in img:
                    img_base64_encode = img.split(',')[1]
                else:
                    img_base64_encode = img

                    # print('image_base64_error',data.get("content").get("image_base64"))
                ans_image_table.append(
                    # dbc.Card(
                        dbc.Col(
                            # dcc.Graph(id=f'annotation_{str(index)}', figure=generator_figure(img_base64_encode,width=400)),
                            # dcc.Graph(id=f'annotation_{str(index)}', figure=b64_to_pil(img_base64_encode,width=400)),
                            # b64_to_pil(img_base64_encode,width=400),
                            # html.Img(src=self.b64_to_pil(img_base64_encode,width=width), className="img_responsive"),
                            html.Img(src=self.b64_to_pil(img_base64_encode,width=width),
                            style={'padding': '0px 0px 0px 0px'},),
                            # html.Img(src=self.b64_to_pil(img_base64_encode,width=width), className="img-fluid"),
                            # width = 6
                        ),
                    )

        if data.get("content").get("table"):
            df_table = pd.DataFrame(json.loads(data.get("content").get("table")))
            if df_table.shape[0]>3:
                df_table = df_table.iloc[0:3,:]
            ans_image_table.append(      
                dbc.Table.from_dataframe(
                    df_table, striped=True, bordered=True, hover=True,size = 'sm',responsive =True,
                    style={'padding': '0px 0px 0px 0px'},
                )
            )
                # )

        # text
        title, text = "", ""
        if data.get("content").get("alarm_title"):
            title += data.get("content").get("alarm_title") + "  "
        if data.get("content").get("message_title"):
            title += data.get("content").get("message_title")

        if data.get("content").get("message_text"):
            text += data.get("content").get("message_text")
        if title:
            ans_title.append(html.H5(title))
        if text:
            ans_text.append(html.P(text))

        # table
        

        # ans.extend(
        #     [
        #         html.Br(),
        #         html.Hr(className="my-2")
        #     ]
        # )
        # button
        if data.get("content").get("buttons"):
            for obj in data.get("content").get("buttons"):
                ans_button.append(
                    dbc.Button(obj.get("title"), href=obj.get("url"), outline=True, color="secondary",  className="me-1",style={'padding': '0px 0px 0px 0px'},),
                    # dbc.Button('无效报警', href=obj.get("url"), outline=True, color="success", className="me-1"),

                    # dbc.Button("无效报警", outline=True, color="warning", className="me-1"),
                    # dbc.Button("添加评论", outline=True, color="secondary", className="me-1"),
                )
        additional_functions_mapping = {
            'valid_alarm_button': "/alarm/respond/valid/",
            'invalid_alarm_button': "/alarm/respond/invalid/",
            'respond_button': "/alarm/comment/",
        }
        if data.get("additional_functions"):
            for name in data.get("additional_functions"):
                ans_button.append(
                    dbc.Button(
                        button_cn_map[name],
                        href=self.SERVER_HOST + additional_functions_mapping.get(name) + feedback_key,
                        outline=True, color="secondary", className="me-1",style={'padding': '0px 0px 0px 0px'},
                    )
                )
        boder_nr = 'default'
        if 'severity' in data:
            severity = data.get("severity")
            if severity <= 20:
                boder_nr = 'green01'
            elif severity <= 40:
                boder_nr = 'green02'
            elif severity <= 60:
                boder_nr = 'yellow'
            elif severity <= 80:
                boder_nr = 'orange'
            elif severity <= 100:
                boder_nr = 'red'


        card = [dbc.Toast(
                    dbc.Row([
                        dbc.Col(
                            ans_image_table,
                            width=9,
                            style={'padding': '0px 2px 0px 15px'},

                        ),
                        dbc.Col([
                            html.P(
                                '报警时间：'+create_time,
                                className="card-text",
                                style={'padding': '0px 10px 0px 0px'},
                            ),
                            html.P(
                                ans_title,
                                className="card-text",
                                style={'padding': '0px 10px 0px 0px'},
                            ),
                            html.P(
                                ans_text,
                                className="card-text",
                                style={'padding': '0px 10px 0px 0px'},
                            ),
                            dbc.Col(ans_button,
                                style={"flexWrap":"nowrap",'padding': '0px 0px 0px 0px'}
                            ),
                            ],
                            width=3,
                            style={'padding': '0px 0px 0px 0px'},
                        )
                        ],
                        style={'padding': '0px 0px 0px 0px'},

                    ),
                    # style={"border":"2px black solid dashed solid double"},
                    # style={"border":"2px black solid   "},
                    className='divBorder_'+boder_nr,
                    # style={'padding': '0px 0px 0px 0px'},
                    style={"maxWidth": "2000px", "width":800,"maxHeight":self.panel_max_height,'padding': '0px 30px 0px 0px'},
                )
            ]

        # card.extend([dcc.Markdown('''---''',style={'padding': '0px 0px 0px 0px'},)])
        card.extend([html.Div(dcc.Markdown('''---''',),style={'padding': '0px 0px 0px 0px'})])


        return card

    def card_output_fun(self):
        # print('开始cardoutput')
        # start_time = '2022-07-05 08:01:06'
        # end_time = '2022-07-05 20:01:06'
        # start_time = '2022-07-06 15:01:06'
        # end_time = '2022-07-06 16:00:06'
        start_time = datetime.datetime.strftime(datetime.datetime.now() - datetime.timedelta(hours=int(self.time_delta)) ,'%Y-%m-%d %H:%M:%S')
        end_time = datetime.datetime.strftime(datetime.datetime.now() ,'%Y-%m-%d %H:%M:%S')
        # start_time = datetime.datetime.strftime(datetime.datetime.now() - datetime.timedelta(hours=100) ,'%Y-%m-%d %H:%M:%S')
        # end_time = datetime.datetime.strftime(datetime.datetime.now() - datetime.timedelta(hours=99),'%Y-%m-%d %H:%M:%S')
        # print('time_delta',self.time_delta)

        # print('start_time',start_time)
        # print('end_time',end_time)
        # print('servicename',self.servicename)

        filter1 = {'username': self.servicename, 'start_time': start_time, 'end_time': end_time, 'read':False}
        order = {'field':'create_time', 'direction':'desc'}

        # print(filter1)
    
        # res = requests.get(f"{SERVER_HOST}/alarm/list", json={"filter": filter}, verify=False).json()
        # res = requests.get(f"{SERVER_HOST}/alarm/list", json={"filter": filter1,'order':order,'page':1, 'size':4}, verify=False).json()
        res = alarm_list_api(self.SERVER_HOST,filter1,order,1,self.alarm_number)

        # print(res)

        if res['data']['records']==[]:

            return [
                    html.Br(),
                    html.Br(),
                    html.P('未查询到报警')
                ],0,0

        else:
            alarm_data = res['data']['records']
            
            card_output = []

            # return [html.P("测试card_output new")]
            count = res['data']['filter_count']
            
            for i in alarm_data:
                create_time = i['createTime']
                data = i["content"]
                feedback_key = i["feedback_key"]
                template = i["content"]["template"]
                # print(template)
                # print('create_time',create_time)

                if template in ('message+image_v2', 'message+image_v1'):
                    card_output.extend(self.screen_msgiv2(data,create_time, feedback_key,width=self.panel_image_width))
                elif template == "message_only_v1":
                    card_output.extend(self.screen_msgiv1(data, create_time,feedback_key))
            
            # card_output.insert(0,
            #                     dbc.Row([
            #                             # dbc.Col(
            #                             #     dbc.Alert("显示最近1小时的4条最新报警，共{}条报警".format(5), color="success"),
            #                             #     width = '100px'
            #                             # ),
            #                             dbc.Col(
            #                                 dbc.Button("显示最近1小时的4条最新报警，共{}条报警, 点击查询更多报警信息".format(5), href='https://www.baidu.com', outline=True, color="success", className="me-1")
            #                                 # width = '40px'
            #                             ),
            #                             dcc.Markdown('''---''')
            #                         ] ,
            #                         style = {'padding': '0px 0px 10px 0px'}                                 
            #                     )
            #                 )
    
            return card_output ,count,len(res['data']['records'])